@@ -1,8 +1,8 @@
 # Paranoid::Debug -- Debug support for paranoid programs
 #
-# (c) 2005, Arthur Corliss <corliss@digitalmages.com>
+# (c) 2005 - 2015, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: Debug.pm,v 0.93 2010/06/03 18:58:30 acorliss Exp $
+# $Id: lib/Paranoid/Debug.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -23,17 +23,14 @@ use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use base qw(Exporter);
 use Paranoid;
 
-($VERSION) = ( q$Revision: 0.93 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 2.02 $ =~ /(\d+(?:\.\d+)+)/sm );
 
-@EXPORT    = qw(PDEBUG pdebug perror pIn pOut psetDebug PDPREFIX);
-@EXPORT_OK = qw(PDEBUG pdebug perror pIn pOut psetDebug PDPREFIX
-    PDLEVEL1 PDLEVEL2 PDLEVEL3 PDLEVEL4 PDMAXINDENT);
-%EXPORT_TAGS = (
-    all => [
-        qw(PDEBUG pdebug perror pIn pOut psetDebug PDPREFIX
-            PDLEVEL1 PDLEVEL2 PDLEVEL3 PDLEVEL4 PDMAXINDENT)
-        ],
-        );
+@EXPORT    = qw(PDEBUG pdebug pIn pOut);
+@EXPORT_OK = (
+    @EXPORT,
+    qw(pderror PDPREFIX PDLEVEL1 PDLEVEL2 PDLEVEL3 PDLEVEL4 PDMAXINDENT)
+    );
+%EXPORT_TAGS = ( all => [@EXPORT_OK], );
 
 use constant PDLEVEL1 => 9;
 use constant PDLEVEL2 => 10;
@@ -83,11 +80,11 @@ use constant PDMAXIND => 60;
         $maxLevel;
     }
 
-    sub perror ($) {
+    sub pderror ($) {
 
         # Purpose:  Print passed string to STDERR
         # Returns:  Return value from print function
-        # Usage:    $rv = perror("Foo!");
+        # Usage:    $rv = pderror("Foo!");
 
         my $msg = shift;
 
@@ -96,9 +93,9 @@ use constant PDMAXIND => 60;
         return print STDERR "$msg\n";
     }
 
-    sub pdebug ($;$) {
+    sub pdebug ($;$@) {
 
-        # Purpose:  Calls perror() if the message level is less than or equal
+        # Purpose:  Calls pderror() if the message level is less than or equal
         #           to the value of PDBEBUG, after prepending the string
         #           returned by the PDPREFIX routine, if defined
         # Returns:  Always returns the passed message, regardless of PDEBUG's
@@ -107,14 +104,50 @@ use constant PDMAXIND => 60;
 
         my $msg    = shift;
         my $level  = shift || 1;
+        my @pfargs = @_;
         my $prefix = PDPREFIX;
+        my ( $n, $np );
+
+        $msg = '' unless defined $msg;
+
+        # Filter message through sprintf if args were passed
+        $n = [ $msg =~ m#(%[\w.]+)#sg ];
+        $np = $n = scalar @$n;
+        if ($n) {
+
+            # Adjust n upwards if we were given more list items than
+            # we see placeholders for in the messsage string
+            $n = scalar @pfargs if @pfargs > $n;
+
+            # Make sure the requisite number of args are translated for undefs
+            while ( $n > 0 ) {
+                $n--;
+                $pfargs[$n] = 'undef' unless defined $pfargs[$n];
+            }
+
+            # Consolidate extra args into the last placeholder's spot
+            if ( scalar @pfargs > $np ) {
+                $n = $np - 1;
+                @pfargs =
+                    ( @pfargs[ 0 .. ( $n - 1 ) ], "@pfargs[$n .. $#pfargs]" );
+            }
+
+            # Filter through sprintf
+            {
+                no warnings;
+                $msg = sprintf( $msg, @pfargs );
+            }
+        }
 
         return $msg if $level > PDEBUG;
 
         # Execute the code block, if that's what it is
         $prefix = &$prefix() if ref($prefix) eq 'CODE';
 
-        perror("$prefix$msg");
+        {
+            no warnings;
+            pderror("$prefix$msg");
+        }
 
         return $msg;
     }
@@ -153,27 +186,6 @@ use constant PDMAXIND => 60;
 
 }
 
-# TODO:  Kill this freaking thing (psetDebug)
-
-sub psetDebug (@) {
-
-    # Purpose:  Set PDEBUG equal to the number of 'v's in '-v...'
-    # Returns:  PDEBUG (after counting v's)
-    # Usage:    psetDebug(@ARGV);
-
-    my @args = @_;
-    my $v;
-
-    # Extract all ^-v+$ arguments
-    $v = join '', grep /^-v+$/sm, @args;
-    $v =~ s/-//smg;
-
-    # Set debug level
-    PDEBUG = length $v;
-
-    return PDEBUG;
-}
-
 1;
 
 __END__
@@ -184,7 +196,7 @@ Paranoid::Debug - Trace message support for paranoid programs
 
 =head1 VERSION
 
-$Id: Debug.pm,v 0.93 2010/06/03 18:58:30 acorliss Exp $
+$Id: lib/Paranoid/Debug.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -206,38 +218,36 @@ $Id: Debug.pm,v 0.93 2010/06/03 18:58:30 acorliss Exp $
     pdebug("leaving w/rv: $rv", 2):
   }
 
-  perror("error msg");
-
-  # Deprecated
-  psetDebug(@ARGV);
+  pderror("error msg");
 
 =head1 DESCRIPTION
 
-The purpose of this module is to provide a barely useful framework to produce
+The purpose of this module is to provide a useful framework to produce
 debugging output.  With this module you can assign a level of detail to pdebug
-statements, and they'll only be displayed when PDEBUG is set to that level or
-higher.  This allows you to have your program produce varying levels of
-debugging output.
+statements, and they'll only be displayed to STDERR when PDEBUG is set to 
+that level or higher.  This allows you to have your program produce varying 
+levels of debugging output.
 
 Using the B<pIn> and B<pOut> functions at the beginning and end of each
 function will cause debugging output to be indented appropriately so you can
 visually see the level of recursion.
 
-B<NOTE:> This module provides a function called B<perror> which conflicts with
-a similar function provided by the B<POSIX> module.  If you use this module
-you should avoid using or importing POSIX's version of this function.
-
 B<NOTE:> All modules within the Paranoid framework use this module.  Their
 debug levels range from 9 and up.  You should use 1 - 8 for your own modules
 or code.
 
+=head2 EXPORT TARGETS
+
+Only I<PDEBUG>, I<pdebug>, I<pIn>, and I<pOut> are exported by default.  All
+other functions and constants can be exported with the I<:all> tag set.
+
 =head1 SUBROUTINES/METHODS
 
 =head2 PDEBUG
 
 B<PDEBUG> is an lvalue subroutine which is initially set to 0, but can be 
-set to any positive integer.  The higher the number the higher the level 
-of pdebug statements are printed.
+set to any positive integer.  The higher the number the more pdebug 
+statements are printed.
 
 =head2 PDMAXINDENT
 
@@ -249,24 +259,25 @@ deep.
 
 =head2 PDPREFIX
 
-B<PDPREFIX> is also an lvalue subroutien and is set by default to a 
+B<PDPREFIX> is also an lvalue subroutine and is set by default to a 
 subroutine that returns as a string the standard prefix for debug 
 messages:
 
   [PID - DLEVEL] Subroutine:
 
-Assigning another subroutine reference to a subroutine can override this 
+Assigning another reference to a subroutine or string can override this 
 behavior.
 
-=head2 perror
+=head2 pderror
 
-  perror("error msg");
+  pderror("error msg");
 
 This function prints the passed message to STDERR.
 
 =head2 pdebug
 
   pdebug("debug statement", 3);
+  pdebug("debug statement: %s %2d %.3f", 3, @values);
 
 This function is called with one mandatory argument (the string to be
 printed), and an optional integer.  This integer is compared against B<PDEBUG>
@@ -277,6 +288,16 @@ single statement to produce debug output and set variables.  For instance:
 
   Paranoid::ERROR = pdebug("Something bad happened!", 3);
 
+As an added benefit you can pass a L<printf> template along with their values
+and they will be handled appropriately.  String values passed as B<undef> will
+be replaced with the literal string "I<undef>".
+
+One deviation from L<printf> allows you to specify a placeholder which can
+gobble up any number of extra arguments while still performing the "I<undef>"
+substitution:
+
+    pdebug("I was passed these values: %s", 3, @values);
+
 =head2 pIn
 
   pIn();
@@ -291,39 +312,23 @@ additional space.
 This function causes all subsequent pdebug messages to be indented by one
 less space.
 
-=head2 psetDebug
-
-  psetDebug(@ARGV);
-
-This function extracts all ^-v+$ arguments from the passed list and counts the
-number of 'v's that result, and sets B<PDEBUG> to that count.  You would
-typically use this by passing @ARGV for command-line programs.
-
-B<NOTE>:  This was a dumb idea of incredible proportions.  As soons as it is
-safe to do so I will kill this function and perform my penance before the gods
-of bitrot.  Consider this deprecated.
-
 =head1 DEPENDENCIES
 
 L<Paranoid>
 
 =head1 BUGS AND LIMITATIONS
 
-B<perror> (and by extension, B<pdebug>) will generate errors if STDERR is
+B<pderror> (and by extension, B<pdebug>) will generate errors if STDERR is
 closed elsewhere in the program.
 
-There is also no upper limit on how much indentation will be used by the
-program, so if you're using B<pIn> in deeply recursive call stacks you can
-expect some overhead due some rather large strings being bandied about.
-
 =head1 AUTHOR
 
-(c) 2005 Arthur Corliss (corliss@digitalmages.com)
+Arthur Corliss (corliss@digitalmages.com)
 
 =head1 LICENSE AND COPYRIGHT
 
 This software is licensed under the same terms as Perl, itself. 
 Please see http://dev.perl.org/licenses/ for more information.
 
-(c) 2005, Arthur Corliss (corliss@digitalmages.com)
+(c) 2005 - 2015, Arthur Corliss (corliss@digitalmages.com)
 
