@@ -1,8 +1,8 @@
 # Paranoid::Network -- Network functions for paranoid programs
 #
-# (c) 2005, Arthur Corliss <corliss@digitalmages.com>
+# (c) 2005 - 2015, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: Network.pm,v 0.69 2012/09/24 22:44:18 acorliss Exp $
+# $Id: lib/Paranoid/Network.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -17,24 +17,26 @@
 
 package Paranoid::Network;
 
-use 5.006;
+use 5.008;
 
 use strict;
 use warnings;
 use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use base qw(Exporter);
+use Paranoid;
 use Paranoid::Debug qw(:all);
 use Paranoid::Network::Socket;
 use Paranoid::Network::IPv4 qw(:all);
 use Paranoid::Network::IPv6 qw(:all);
-use Carp;
 
-($VERSION) = ( q$Revision: 0.69 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 2.02 $ =~ /(\d+(?:\.\d+)+)/sm );
 
-@EXPORT    = qw(ipInNetwork hostInDomain extractIPs netIntersect);
-@EXPORT_OK = qw(ipInNetwork hostInDomain extractIPs netIntersect);
-%EXPORT_TAGS =
-    ( all => [qw(ipInNetwork hostInDomain extractIPs netIntersect)], );
+@EXPORT      = qw(ipInNetworks hostInDomains extractIPs netIntersect);
+@EXPORT_OK   = ( @EXPORT, qw(NETMATCH HOSTNAME_REGEX) );
+%EXPORT_TAGS = ( all => [@EXPORT_OK], );
+
+use constant HOSTNAME_REGEX =>
+    qr#(?:[a-z0-9][a-z0-9\-]*)(?:\.[a-z0-9][a-z0-9\-]*)*\.?#s;
 
 #####################################################################
 #
@@ -42,42 +44,53 @@ use Carp;
 #
 #####################################################################
 
-sub ipInNetwork ($@) {
+{
+
+    my $lmatch;
+
+    sub NETMATCH : lvalue {
+        $lmatch;
+    }
+
+}
+
+sub ipInNetworks {
 
     # Purpose:  Checks to see if the IP occurs in the passed list of IPs and
     #           networks
     # Returns:  True (1) if the IP occurs, False (0) otherwise
-    # Usage:    $rv = ipInNetwork($ip, @networks);
+    # Usage:    $rv = ipInNetworks($ip, @networks);
 
     my $ip       = shift;
-    my $i        = defined $ip ? $ip : 'undef';
     my @networks = grep {defined} @_;
     my $rv       = 0;
     my ( $family, @tmp );
 
-    pdebug( "entering w/($i)(@networks)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $ip, @networks );
     pIn();
 
+    NETMATCH = undef;
+
     # Validate arguments
     if ( defined $ip ) {
 
         # Extract IPv4 address from IPv6 encoding
-        $ip =~ s/^::ffff:(@{[ IPV4REGEX ]})$/$1/smio;
+        $ip =~ s/^::ffff:(@{[ IPV4REGEX ]})$/$1/sio;
 
         # Check for IPv6 support
         if ( has_ipv6() or $] >= 5.012 ) {
 
             pdebug( 'Found IPv4/IPv6 support', PDLEVEL2 );
             $family =
-                  $ip =~ m/^@{[ IPV4REGEX ]}$/smo ? AF_INET()
-                : $ip =~ m/^@{[ IPV6REGEX ]}$/smo ? AF_INET6()
-                :                                   undef;
+                  $ip =~ m/^@{[ IPV4REGEX ]}$/so ? AF_INET()
+                : $ip =~ m/^@{[ IPV6REGEX ]}$/so ? AF_INET6()
+                :                                  undef;
 
         } else {
 
             pdebug( 'Found only IPv4 support', PDLEVEL2 );
             $family = AF_INET()
-                if $ip =~ m/^@{[ IPV4REGEX ]}$/smo;
+                if $ip =~ m/^@{[ IPV4REGEX ]}$/so;
         }
     }
 
@@ -86,12 +99,11 @@ sub ipInNetwork ($@) {
         # Filter out non-family data from @networks
         @networks = grep {
             $family == AF_INET()
-                ? m#^@{[ IPV4CIDRRGX ]}$#smo
-                : m#^@{[ IPV6CIDRRGX ]}$#smo
+                ? m#^(?:@{[ IPV4CIDRRGX ]}|@{[ IPV4REGEX ]})$#so
+                : m#^(?:@{[ IPV6CIDRRGX ]}|@{[ IPV6REGEX ]})$#so
         } @networks;
 
-        pdebug( "networks to compare: @{[ join ', ', @networks ]}",
-            PDLEVEL2 );
+        pdebug( 'networks to compare: %s', PDLEVEL2, @networks );
 
         # Start comparisons
         foreach (@networks) {
@@ -99,6 +111,7 @@ sub ipInNetwork ($@) {
                 ? ipv4NetIntersect( $ip, $_ )
                 : ipv6NetIntersect( $ip, $_ )
                 ) {
+                NETMATCH = $_;
                 $rv = 1;
                 last;
             }
@@ -106,94 +119,101 @@ sub ipInNetwork ($@) {
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub hostInDomain ($@) {
+sub hostInDomains {
 
     # Purpose:  Checks to see if the host occurs in the list of domains
     # Returns:  True (1) if the host occurs, False (0) otherwise
-    # Usage:    $rv = hostInDomain($hostname, @domains);
+    # Usage:    $rv = hostInDomains($hostname, @domains);
 
     my $host    = shift;
     my @domains = @_;
     my $rv      = 0;
     my $domain;
 
-    # Validate arguments
-    croak 'Mandatory first argument must be a defined and valid hostname'
-        unless defined $host && $host =~ /^(?:[\w\-]+\.)*[\w\-]+$/sm;
-
-    pdebug( "entering w/($host)(@domains)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $host, @domains );
     pIn();
 
-    # Filter out non-domains
-    @domains = grep { defined $_ && m/^(?:[\w\-]+\.)*[\w\-]+$/sm } @domains;
+    NETMATCH = undef;
 
-    # Start the comparison
-    if (@domains) {
-        foreach $domain (@domains) {
-            if ( $host =~ /^(?:[\w\-]+\.)*\Q$domain\E$/smi ) {
-                $rv = 1;
-                last;
+    if ( defined $host and $host =~ /^@{[ HOSTNAME_REGEX ]}$/so ) {
+
+        # Filter out non-domains
+        @domains =
+            grep { defined $_ && m/^@{[ HOSTNAME_REGEX ]}$/so } @domains;
+
+        # Start the comparison
+        if (@domains) {
+            foreach $domain (@domains) {
+                if ( $host =~ /^(?:[\w\-]+\.)*\Q$domain\E$/si ) {
+                    NETMATCH = $domain;
+                    $rv = 1;
+                    last;
+                }
             }
         }
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub extractIPs (@) {
+sub extractIPs {
 
     # Purpose:  Extracts IPv4/IPv6 addresses from arbitrary text.
     # Returns:  List containing extracted IP addresses
-    # Usage:    @ips = extractIP($string1, $string2);
+    # Usage:    @ips = extractIPs($string1, $string2);
 
-    my @strings = grep {defined} @_;
+    my @strings = @_;
     my ( $string, @ips, $ip, @tmp, @rv );
 
-    pdebug( "entering w/(@strings)", PDLEVEL1 );
+    pdebug( 'entering w/%d strings', PDLEVEL1, scalar @strings );
     pIn();
 
     foreach $string (@strings) {
+        next unless defined $string;
 
         # Look for IPv4 addresses
-        @ips = ( $string =~ /(@{[ IPV4REGEX ]})/smog );
+        @ips = ( $string =~ /(@{[ IPV4CIDRRGX ]}|@{[ IPV4REGEX ]})/sog );
 
         # Validate them by filtering through inet_aton
         foreach $ip (@ips) {
-            push @rv, $ip if defined inet_aton($ip);
+            @tmp = split m#/#s, $ip;
+            push @rv, $ip if defined inet_aton( $tmp[0] );
         }
 
         # If Socket6 is present or we have Perl 5.14 or higher we'll check
         # for IPv6 addresses
         if ( has_ipv6() or $] >= 5.012 ) {
 
-            @ips = ( $string =~ m/(@{[ IPV6REGEX ]})/smogix );
+            @ips = ( $string =~
+                    m/(@{[ IPV6CIDRRGX ]}|@{[ IPV6REGEX ]})/sogix );
 
             # Filter out addresses with more than one ::
-            @ips = grep { scalar(m/(::)/smg) <= 1 } @ips;
+            @ips = grep { scalar(m/(::)/sg) <= 1 } @ips;
 
             # Validate remaining addresses with inet_pton
             foreach $ip (@ips) {
+                @tmp = split m#/#s, $ip;
                 push @rv, $ip
-                    if defined inet_pton( AF_INET6(), $ip );
+                    if defined inet_pton( AF_INET6(), $tmp[0] );
             }
         }
     }
 
     pOut();
-    pdebug( "leaving w/rv: @rv)", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, @rv );
 
     return @rv;
 }
 
-sub netIntersect (@) {
+sub netIntersect {
 
     # Purpose:  Tests whether network address ranges intersect
     # Returns:  Integer, denoting whether an intersection exists, and what
@@ -207,25 +227,24 @@ sub netIntersect (@) {
 
     my $target = shift;
     my $dest   = shift;
-    my $t      = defined $target ? $target : 'undef';
-    my $d      = defined $dest ? $dest : 'undef';
     my $rv     = 0;
 
-    pdebug( "entering w/$t, $d", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $target, $dest );
     pIn();
 
     if ( defined $target and defined $dest ) {
-        if ( $target =~ m/^@{[ IPV4CIDRRGX ]}$/sm ) {
+        if ( $target =~ m/^(?:@{[ IPV4CIDRRGX ]}|@{[ IPV4REGEX ]})$/s ) {
             $rv = ipv4NetIntersect( $target, $dest );
-        } elsif ( $target =~ m/^@{[ IPV6CIDRRGX ]}$/smi ) {
+        } elsif ( $target =~ m/^(?:@{[ IPV6CIDRRGX ]}|@{[ IPV6REGEX ]})$/si )
+        {
             $rv = ipv6NetIntersect( $target, $dest )
                 if has_ipv6()
                     or $] >= 5.012;
         } else {
             pdebug(
-                "target string ($target) doesn't seem to match"
+                'target string (%s) doesn\'t seem to match '
                     . 'an IP/network address',
-                PDLEVEL1
+                PDLEVEL1, $target
                 );
         }
     } else {
@@ -233,7 +252,7 @@ sub netIntersect (@) {
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
@@ -248,15 +267,17 @@ Paranoid::Network - Network functions for paranoid programs
 
 =head1 VERSION
 
-$Id: Network.pm,v 0.69 2012/09/24 22:44:18 acorliss Exp $
+$Id: lib/Paranoid/Network.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 
 =head1 SYNOPSIS
 
   use Paranoid::Network;
 
-  $rv  = ipInNetwork($ip, @networks);
-  $rv  = hostInDomain($host, @domains);
-  @ips = extractIP($string1, $string2);
+  $rv  = ipInNetworks($ip, @networks);
+  $rv  = hostInDomains($host, @domains);
+  $cidr = Paranoid::Network::NETMATCH();
+  @ips = extractIPs($string1);
+  @ips = extractIPs(@lines);
   $rv  = netIntersect( $cidr1, $cidr2 );
 
 =head1 DESCRIPTION
@@ -266,11 +287,13 @@ data.  It attempts to be IPv4/IPv6 agnostic, assuming IPv6 support is present.
 Due to the gradual introduction of IPv6 support into Perl there may be
 caveats.  Please consult L<Paranoid::Network::Socket> for more details.
 
+I<NETMATCH> and I<HOSTNAME_REGEX> are not exported by default.
+
 =head1 SUBROUTINES/METHODS
 
-=head2 ipInNetwork
+=head2 ipInNetworks
 
-  $rv = ipInNetwork($ip, @networks);
+  $rv = ipInNetworks($ip, @networks);
 
 This function checks the passed IP (in string format) against each of the 
 networks or IPs in the list and returns true if there's a match.  The list of 
@@ -301,11 +324,11 @@ Just list IPv4 as IPv4, IPv6 as IPv6, and this routine will convert
 IPv6-encoded IPv4 addresses automatically.  This would make the following test
 return a true value:
 
-  ipInNetwork( '::ffff:192.168.0.5', '192.168.0.0/24' );
+  ipInNetworks( '::ffff:192.168.0.5', '192.168.0.0/24' );
 
 but
 
-  ipInNetwork( '::ffff:192.168.0.5', '::ffff:192.168.0.0/120' );
+  ipInNetworks( '::ffff:192.168.0.5', '::ffff:192.168.0.0/120' );
 
 return a false value.  This may seem counter intuitive, but it simplifies
 things in (my alternate) reality.
@@ -313,18 +336,34 @@ things in (my alternate) reality.
 Please note that this automatic conversion only applies to the B<IP> argument,
 not to any member of the network array.
 
-=head2 hostInDomain
+=head2 hostInDomains
 
-  $rv = hostInDomain($host, @domains);
+  $rv = hostInDomains($host, @domains);
 
 This function checks the passed hostname (fully qualified) against each 
 of the domains in the list and returns true if there's a match.  None of the
 domains should have the preceding '.' (i.e., 'foo.com' rather than 
 '.foo.com').
 
+=head2 NETMATCH
+
+  $cidr = Paranoid::Network::NETMATCH();
+
+This stores the IP, network address, or domain that matched in 
+I<ipInNetworks> or I<hostInDomains>.  This returns B<undef> if any
+function call fails to make a match.
+
+=head2 HOSTNAME_REGEX
+
+    $rv = $hostname =~ /^@{[ HOSTNAME_REGEX ]}$/so;
+
+This constant is just a regex meant to be a basic sanity check for appropriate
+hostnames.  It is probably overly strict in accordance with outdated RFCs.
+
 =head2 extractIPs
 
-    @ips = extractIP($string1, $string2);
+    @ips = extractIPs($string1);
+    @ips = extractIPs(@lines);
 
 This function extracts IPv4/IPv6 addresses from arbitrary text.  IPv6 support
 is contingent upon the presence of proper support (please see
@@ -341,7 +380,7 @@ with the overhead of B<Paranoid> (with debugging and I<loadModule> calls for
 Socket6 and what-not) it seems that this is an order of a magnitude faster
 than doing a pure regex extraction & validation of IPv6 addresses.
 
-B<NOTE:> Like the B<ipInNetwork> function we filter out IPv4 addresses encoded
+B<NOTE:> Like the B<ipInNetworks> function we filter out IPv4 addresses encoded
 as IPv6 addresses since that address is already returned as a pure IPv4
 address.
 
@@ -364,6 +403,10 @@ L<Paranoid>
 
 =item o
 
+L<Paranoid::Debug>
+
+=item o
+
 L<Paranoid::Network::Socket>
 
 =item o
@@ -387,5 +430,5 @@ Arthur Corliss (corliss@digitalmages.com)
 This software is licensed under the same terms as Perl, itself. 
 Please see http://dev.perl.org/licenses/ for more information.
 
-(c) 2005, Arthur Corliss (corliss@digitalmages.com)
+(c) 2005 - 2015, Arthur Corliss (corliss@digitalmages.com)
 
