@@ -1,8 +1,8 @@
 # Paranoid::Process -- Process management support for paranoid programs
 #
-# (c) 2005, Arthur Corliss <corliss@digitalmages.com>
+# (c) 2005 - 2015, Arthur Corliss <corliss@digitalmages.com>
 #
-# $Id: Process.pm,v 0.101 2010/05/10 04:43:30 acorliss Exp $
+# $Id: lib/Paranoid/Process.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -17,7 +17,7 @@
 
 package Paranoid::Process;
 
-use 5.006;
+use 5.008;
 
 use strict;
 use warnings;
@@ -28,23 +28,18 @@ use Paranoid::Debug qw(:all);
 use POSIX qw(getuid setuid setgid WNOHANG setsid);
 use Carp;
 
-($VERSION) = ( q$Revision: 0.101 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 2.02 $ =~ /(\d+(?:\.\d+)+)/sm );
 
 @EXPORT    = qw(switchUser daemonize);
-@EXPORT_OK = qw(MAXCHILDREN      childrenCount   installChldHandler
-    sigchld          pfork           ptranslateUser
-    ptranslateGroup  switchUser      pcapture
-    daemonize);
+@EXPORT_OK = (
+    @EXPORT, qw(MAXCHILDREN childrenCount installChldHandler
+        sigchld pfork ptranslateUser ptranslateGroup pcapture
+        ) );
 %EXPORT_TAGS = (
-    all => [
-        qw(MAXCHILDREN      childrenCount   installChldHandler
-            sigchld          pfork           ptranslateUser
-            ptranslateGroup  switchUser      pcapture
-            daemonize)
-        ],
+    all   => [@EXPORT_OK],
     pfork => [
-        qw(MAXCHILDREN      childrenCount   installChldHandler
-            sigchld          pfork           daemonize)
+        qw(MAXCHILDREN childrenCount installChldHandler
+            sigchld pfork daemonize)
         ],
         );
 
@@ -68,27 +63,25 @@ use Carp;
 
         $maxChildren;
     }
-    sub childrenCount () { return $numChildren }
-    sub _incrChildren () { $numChildren++ }
-    sub _decrChildren () { $numChildren-- }
+    sub childrenCount { return $numChildren }
+    sub _incrChildren { $numChildren++ }
+    sub _decrChildren { $numChildren-- }
 
-    sub installChldHandler ($) {
+    sub installChldHandler (\&) {
 
         # Purpose:  Installs a code reference to execute whenever a child
         #           exits
         # Returns:  True (1)
-        # Usage:    installChldHandler(\&foo);
+        # Usage:    installChldHandler(&foo);
 
         $chldRef = shift;
 
-        croak 'installChldHandler passed no sub ref!'
-            unless defined $chldRef && ref($chldRef) eq 'CODE';
         return 1;
     }
-    sub _chldHandler () { return $chldRef }
+    sub _chldHandler { return $chldRef }
 }
 
-sub sigchld () {
+sub sigchld {
 
     # Purpose:  Default signal handler for SIGCHLD
     # Returns:  True (1)
@@ -106,7 +99,7 @@ sub sigchld () {
         $pid = waitpid -1, WNOHANG;
         if ( $pid > 0 ) {
             _decrChildren();
-            pdebug( "child $pid reaped w/rv: $?", PDLEVEL1 );
+            pdebug( 'child %d reaped w/rv: %s', PDLEVEL1, $pid, $? );
 
             # Call the user's sig handler if defined
             &$sref( $pid, $? ) if defined $sref;
@@ -119,7 +112,7 @@ sub sigchld () {
     return 1;
 }
 
-sub daemonize () {
+sub daemonize {
 
     # Purpose:  Daemonizes process and disassociates with the terminal
     # Returns:  True unless there are errors.
@@ -138,7 +131,7 @@ sub daemonize () {
     if ( defined $pid ) {
 
         # Fork was successful, close parent file descriptors
-        $rv = open(STDIN, '/dev/null') and open(STDOUT, '>/dev/null');
+        $rv = open( STDIN, '/dev/null' ) and open( STDOUT, '>/dev/null' );
 
         # Create a new process group
         unless ($rv) {
@@ -150,17 +143,17 @@ sub daemonize () {
 
     } else {
         Paranoid::ERROR =
-            pdebug( "Failed to daemonize process: $!", PDLEVEL1 );
+            pdebug( 'Failed to daemonize process: %s', PDLEVEL1, $! );
         $rv = 0;
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub pfork () {
+sub pfork {
 
     # Purpose:  Replacement for Perl's fork function.  Blocks until a child
     #           exists if MAXCHILDREN is exceeded.
@@ -169,7 +162,7 @@ sub pfork () {
     # Usage:    $rv = pfork();
 
     my $max = MAXCHILDREN();
-    my ( $rv, $rvarg );
+    my $rv;
 
     pdebug( 'entering', PDLEVEL1 );
     pIn();
@@ -182,77 +175,72 @@ sub pfork () {
     # Fork and return
     $rv = fork;
     _incrChildren() if defined $rv;
-    $rvarg = defined $rv ? $rv : 'undef';
 
     pOut();
-    pdebug( "leaving w/rv: $rvarg", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub ptranslateUser ($) {
+sub ptranslateUser {
 
     # Purpose:  Translates a string account name into the UID
     # Returns:  UID if found, undef if not
     # Usage:    $uid = ptranslateUser($user);
 
     my $user = shift;
-    my ( $uuid, @pwentry, $rv, $rvarg );
+    my ( $uuid, @pwentry, $rv );
 
-    # Validate arguments
-    croak 'Mandatory first argument must be a defined username'
-        unless defined $user;
-
-    pdebug( "entering w/($user)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)', PDLEVEL1, $user );
     pIn();
 
-    setpwent;
-    do {
-        @pwentry = getpwent;
-        $uuid = $pwentry[2] if @pwentry && $user eq $pwentry[0];
-    } until defined $uuid || !@pwentry;
-    endpwent;
-    $rv = $uuid if defined $uuid;
-    $rvarg = defined $rv ? $rv : 'undef';
+    if ( defined $user and length $user ) {
+
+        setpwent;
+        do {
+            @pwentry = getpwent;
+            $uuid = $pwentry[2] if @pwentry && $user eq $pwentry[0];
+        } until defined $uuid || !@pwentry;
+        endpwent;
+        $rv = $uuid if defined $uuid;
+    }
 
     pOut();
-    pdebug( "leaving w/rv: $rvarg", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub ptranslateGroup ($) {
+sub ptranslateGroup {
 
     # Purpose:  Translates a string group name into the UID
     # Returns:  GID if found, undef if not
     # Usage:    $gid = ptranslateGroup($group);
 
     my $group = shift;
-    my ( $ugid, @pwentry, $rv, $rvarg );
+    my ( $ugid, @pwentry, $rv );
 
-    # Validate arguments
-    croak 'Mandatory first argument must be a defined group name'
-        unless defined $group;
-
-    pdebug( "entering w/($group)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)', PDLEVEL1, $group );
     pIn();
 
-    setgrent;
-    do {
-        @pwentry = getgrent;
-        $ugid = $pwentry[2] if @pwentry && $group eq $pwentry[0];
-    } until defined $ugid || !@pwentry;
-    endgrent;
-    $rv = $ugid if defined $ugid;
-    $rvarg = defined $rv ? $rv : 'undef';
+    if ( defined $group and length $group ) {
+
+        setgrent;
+        do {
+            @pwentry = getgrent;
+            $ugid = $pwentry[2] if @pwentry && $group eq $pwentry[0];
+        } until defined $ugid || !@pwentry;
+        endgrent;
+        $rv = $ugid if defined $ugid;
+    }
 
     pOut();
-    pdebug( "leaving w/rv: $rvarg", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub switchUser ($;$) {
+sub switchUser {
 
     # Purpose:  Switches to the user/group specified
     # Returns:  True (1) if successful, False (0) if not
@@ -261,8 +249,6 @@ sub switchUser ($;$) {
 
     my $user  = shift;
     my $group = shift;
-    my $uarg  = defined $user ? $user : 'undef';
-    my $garg  = defined $group ? $group : 'undef';
     my $rv    = 1;
     my ( @pwentry, $duid, $dgid );
 
@@ -270,29 +256,29 @@ sub switchUser ($;$) {
     croak 'Mandatory argument of either user or group must be passed'
         unless defined $user || defined $group;
 
-    pdebug( "entering w/($uarg)($garg)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)', PDLEVEL1, $user, $group );
     pIn();
 
     # First switch the group
     if ( defined $group ) {
 
         # Look up named group
-        unless ( $group =~ /^\d+$/sm ) {
+        unless ( $group =~ /^\d+$/s ) {
             $dgid = ptranslateGroup($group);
             unless ( defined $dgid ) {
-                Paranoid::ERROR =
-                    pdebug( "couldn't identify group ($group)", PDLEVEL1 );
+                Paranoid::ERROR = pdebug( 'couldn\'t identify group (%s)',
+                    PDLEVEL1, $group );
                 $rv = 0;
             }
         }
 
         # Switch to group
         if ($rv) {
-            pdebug( "switching to GID $dgid", PDLEVEL2 );
+            pdebug( 'switching to GID %s', PDLEVEL2, $dgid );
             unless ( setgid($dgid) ) {
                 Paranoid::ERROR =
-                    pdebug( "couldn't switch to group ($group): $!",
-                    PDLEVEL1 );
+                    pdebug( 'couldn\'t switch to group (%s): %s',
+                    PDLEVEL1, $group, $! );
                 $rv = 0;
             }
         }
@@ -302,38 +288,38 @@ sub switchUser ($;$) {
     if ( $rv && defined $user ) {
 
         # Look up named user
-        unless ( $user =~ /^\d+$/sm ) {
+        unless ( $user =~ /^\d+$/s ) {
             $duid = ptranslateUser($user);
             unless ( defined $duid ) {
                 Paranoid::ERROR =
-                    pdebug( "couldn't identify user ($user)", PDLEVEL1 );
+                    pdebug( 'couldn\'t identify user (%s)', PDLEVEL1, $user );
                 $rv = 0;
             }
         }
 
         # Switch to user
         if ($rv) {
-            pdebug( "switching to UID $duid", PDLEVEL2 );
+            pdebug( 'switching to UID %s', PDLEVEL2, $duid );
             unless ( setuid($duid) ) {
-                Paranoid::ERROR =
-                    pdebug( "couldn't switch to user ($user): $!", PDLEVEL1 );
+                Paranoid::ERROR = pdebug( 'couldn\'t switch to user (%s): %s',
+                    PDLEVEL1, $user, $! );
                 $rv = 0;
             }
         }
     }
 
     pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
+    pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
 
     return $rv;
 }
 
-sub pcapture ($$$) {
+sub pcapture ($\$\$) {
 
     # Purpose:  Captures the output and exit code of the specified shell
     #           command.  Output incorporates STDERR via redirection.
     # Returns:  True (1) if command exits cleanly, False (0) otherwise
-    # Usage:    $rv = pcapture($cmd, \$crv, \$out);
+    # Usage:    $rv = pcapture($cmd, $crv, $out);
 
     my $cmd  = shift;
     my $cref = shift;
@@ -341,64 +327,60 @@ sub pcapture ($$$) {
     my $rv   = -1;
     my ( $sigchld, $cored, $signal );
 
-    # Validate arguments
-    croak 'Mandatory first argument must be a defined shell command string'
-        unless defined $cmd;
-    croak 'Mandatory second argument must be a scalar reference'
-        unless defined $cref && ref $cref eq 'SCALAR';
-    croak 'Mandatory third argument must be a scalar reference'
-        unless defined $oref && ref $oref eq 'SCALAR';
-
-    pdebug( "entering w/($cmd)($cref)($oref)", PDLEVEL1 );
+    pdebug( 'entering w/(%s)(%s)(%s)', PDLEVEL1, $cmd, $cref, $oref );
     pIn();
 
-    # Massage the command string
-    $cmd = "( $cmd ) 2>&1";
+    if ( defined $cmd ) {
 
-    # Backup SIGCHLD handler and set it to something safe
-    if ( defined $SIG{CHLD} ) {
-        $sigchld = $SIG{CHLD};
-        $SIG{CHLD} = sub {1};
+        # Massage the command string
+        $cmd = "( $cmd ) 2>&1";
+
+        # Backup SIGCHLD handler and set it to something safe
+        if ( defined $SIG{CHLD} ) {
+            $sigchld = $SIG{CHLD};
+            $SIG{CHLD} = sub {1};
+        }
+
+        # Execute and snarf the output
+        pdebug( 'executing command', PDLEVEL2 );
+        $$oref  = `$cmd`;
+        $$cref  = $?;
+        $cored  = $$cref & 128;
+        $signal = $$cref & 127;
+        pdebug( 'command exited with raw rv: %s', PDLEVEL2, $$cref );
+
+        # Restore SIGCHLD handler
+        $SIG{CHLD} = $sigchld if defined $SIG{CHLD};
+
+        # Check the return value
+        if ( $$cref == -1 or $$cref == 32512 ) {
+
+            # Command failed to execute
+            Paranoid::ERROR =
+                pdebug( 'command failed to execute: %s', PDLEVEL1, $! );
+            $rv = -1;
+
+        } elsif ($signal) {
+
+            # Exited with signal (and core?)
+            Paranoid::ERROR =
+                pdebug( 'command died with signal: %s', PDLEVEL1, $signal );
+            pdebug( "command exited with core dump", PDLEVEL1 ) if $cored;
+            $rv = -1;
+
+        } else {
+
+            # Command exited normally
+            $$cref >>= 8;
+            $rv = $$cref == 0 ? 1 : 0;
+            pdebug( 'command exited with rv: %s', PDLEVEL1, $$cref );
+        }
+
+        pOut();
+        pdebug( 'leaving w/rv: %s', PDLEVEL1, $rv );
+
+        return $rv;
     }
-
-    # Execute and snarf the output
-    pdebug( 'executing command', PDLEVEL2 );
-    $$oref  = `$cmd`;
-    $$cref  = $?;
-    $cored  = $$cref & 128;
-    $signal = $$cref & 127;
-    pdebug( "command exited with raw rv: $$cref", PDLEVEL2 );
-
-    # Restore SIGCHLD handler
-    $SIG{CHLD} = $sigchld if defined $SIG{CHLD};
-
-    # Check the return value
-    if ( $$cref == -1 or $$cref == 32512 ) {
-
-        # Command failed to execute
-        Paranoid::ERROR = pdebug( "command failed to execute: $!", PDLEVEL1 );
-        $rv = -1;
-
-    } elsif ($signal) {
-
-        # Exited with signal (and core?)
-        Paranoid::ERROR =
-            pdebug( "command died with signal: $signal", PDLEVEL1 );
-        pdebug( "command exited with core dump", PDLEVEL1 ) if $cored;
-        $rv = -1;
-
-    } else {
-
-        # Command exited normally
-        $$cref >>= 8;
-        $rv = $$cref == 0 ? 1 : 0;
-        pdebug( "command exited with rv: $$cref", PDLEVEL1 );
-    }
-
-    pOut();
-    pdebug( "leaving w/rv: $rv", PDLEVEL1 );
-
-    return $rv;
 }
 
 1;
@@ -411,7 +393,7 @@ Paranoid::Process - Process Management Functions
 
 =head1 VERSION
 
-$Id: Process.pm,v 0.101 2010/05/10 04:43:30 acorliss Exp $
+$Id: lib/Paranoid/Process.pm, 2.02 2016/08/09 17:39:27 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -423,14 +405,14 @@ $Id: Process.pm,v 0.101 2010/05/10 04:43:30 acorliss Exp $
 
   $SIG{CHLD} = \&sigchld;
   $count = childrenCount();
-  installChldHandler($sub);
+  installChldHandler(&cleanup);
   $rv = pfork();
 
   $uid = ptranslateUser("foo");
   $gid = ptranslateGroup("foo");
   $rv = switchUser($user, $group);
 
-  $rv = pcapture($cmd, \$crv, \$out);
+  $rv = pcapture($cmd, $crv, $out);
 
 =head1 DESCRIPTION
 
@@ -460,7 +442,7 @@ This function returns the current number of children spawned by B<pfork>.
 
 =head2 installChldHandler
 
-  installChldHandler($sub);
+  installChldHandler(&cleanup);
 
 This function takes a reference to a subroutine.  If used the subroutine will
 be called every time a child exits.  That subroutine will be called with the
@@ -518,7 +500,7 @@ only the user argument.
 
 =head2 pcapture
 
-  $rv = pcapture($cmd, \$crv, \$out);
+  $rv = pcapture($cmd, $crv, $out);
 
 This function executes the passed shell command and returns one of the following
 three values:
@@ -529,8 +511,8 @@ three values:
    0    Command executed but exited with a non-0 RV
    1    Command executed and exited with a 0 RV
 
-The actual return value is populated in the passed scalar reference, while all
-STDERR/STDOUT output is stored in the last scalar reference.  Any errors
+The actual return value is populated in the passed scalar, while all
+command output (including STDERR) is stored in the next scalar.  Any errors
 executing the command will have the error string stored in B<Paranoid::ERROR>.
 
 If the command exited cleanly it will automatically be bit shifted eight
@@ -548,6 +530,10 @@ your script to exit under taint mode.
 
 =item o
 
+L<Carp>
+
+=item o
+
 L<Paranoid>
 
 =item o
@@ -587,7 +573,7 @@ For instance, to track the children's history in the parent:
     push(@chistory, [$cpid, $cexit]);
   }
 
-  installChldHandler(\&recordChild);
+  installChldHandler(&recordChild);
   for (1 .. 5) {
     unless ($pid = pfork()) {
       # ....
@@ -617,5 +603,5 @@ Arthur Corliss (corliss@digitalmages.com)
 This software is licensed under the same terms as Perl, itself. 
 Please see http://dev.perl.org/licenses/ for more information.
 
-(c) 2005, Arthur Corliss (corliss@digitalmages.com)
+(c) 2005 - 2015, Arthur Corliss (corliss@digitalmages.com)
 
